@@ -63,9 +63,11 @@
 
 #![cfg_attr(not(feature = "std"), no_std)]
 
+use self::NullableResult::*;
 use core::{
     fmt::Debug,
     iter::{FilterMap, FromIterator, FusedIterator},
+    ops::Deref,
 };
 
 #[derive(Debug, Copy, Clone, Ord, PartialOrd, Eq, PartialEq, Hash)]
@@ -78,7 +80,7 @@ pub enum NullableResult<T, E> {
 
 impl<T, E> Default for NullableResult<T, E> {
     fn default() -> Self {
-        NullableResult::None
+        None
     }
 }
 
@@ -99,12 +101,67 @@ impl<T, E: Debug> NullableResult<T, E> {
     }
 }
 
+impl<T: Default, E> NullableResult<T, E> {
+    #[inline]
+    pub fn unwrap_or_default(self) -> T {
+        match self {
+            Ok(item) => item,
+            _ => T::default(),
+        }
+    }
+}
+
+impl<T: Copy, E> NullableResult<&'_ T, E> {
+    #[inline]
+    pub fn copied(self) -> NullableResult<T, E> {
+        self.map(|&item| item)
+    }
+}
+
+impl<T: Copy, E> NullableResult<&'_ mut T, E> {
+    #[inline]
+    pub fn copied(self) -> NullableResult<T, E> {
+        self.map(|&mut item| item)
+    }
+}
+
+impl<T: Clone, E> NullableResult<&'_ T, E> {
+    pub fn cloned(self) -> NullableResult<T, E> {
+        self.map(|item| item.clone())
+    }
+}
+
+impl<T: Clone, E> NullableResult<&'_ mut T, E> {
+    pub fn cloned(self) -> NullableResult<T, E> {
+        self.map(|item| item.clone())
+    }
+}
+
+impl<T: Deref, E> NullableResult<T, E> {
+    #[inline]
+    pub fn as_deref(&self) -> NullableResult<&T::Target, &E> {
+        match self {
+            Ok(item) => Ok(item.deref()),
+            Err(err) => Err(err),
+            None => None,
+        }
+    }
+}
+
 impl<T, E> NullableResult<T, E> {
+    #[inline]
+    pub fn expect(self, msg: &str) -> T {
+        match self {
+            Ok(item) => item,
+            _ => panic!("{}", msg),
+        }
+    }
+
     /// Returns the contained value if it's `Ok`, returns `item` otherwise.
     #[inline]
     pub fn unwrap_or(self, item: T) -> T {
         match self {
-            NullableResult::Ok(item) => item,
+            Ok(item) => item,
             _ => item,
         }
     }
@@ -114,7 +171,7 @@ impl<T, E> NullableResult<T, E> {
     #[inline]
     pub fn unwrap_or_else<F: FnOnce() -> T>(self, f: F) -> T {
         match self {
-            NullableResult::Ok(item) => item,
+            Ok(item) => item,
             _ => f(),
         }
     }
@@ -124,8 +181,8 @@ impl<T, E> NullableResult<T, E> {
     #[inline]
     pub fn option(self) -> Option<T> {
         match self {
-            NullableResult::Ok(item) => Some(item),
-            NullableResult::Err(_) | NullableResult::None => None,
+            Ok(item) => Some(item),
+            Err(_) | None => Option::None,
         }
     }
 
@@ -146,9 +203,9 @@ impl<T, E> NullableResult<T, E> {
     #[inline]
     pub fn result(self, err: E) -> Result<T, E> {
         match self {
-            NullableResult::Ok(item) => Ok(item),
-            NullableResult::Err(err) => Err(err),
-            NullableResult::None => Err(err),
+            Ok(item) => Result::Ok(item),
+            Err(err) => Result::Err(err),
+            None => Result::Err(err),
         }
     }
 
@@ -157,9 +214,9 @@ impl<T, E> NullableResult<T, E> {
     #[inline]
     pub fn result_with<F: FnOnce() -> E>(self, f: F) -> Result<T, E> {
         match self {
-            NullableResult::Ok(item) => Ok(item),
-            NullableResult::Err(err) => Err(err),
-            NullableResult::None => Err(f()),
+            Ok(item) => Result::Ok(item),
+            Err(err) => Result::Err(err),
+            None => Result::Err(f()),
         }
     }
 
@@ -167,9 +224,9 @@ impl<T, E> NullableResult<T, E> {
     #[inline]
     pub fn map<U, F: FnOnce(T) -> U>(self, f: F) -> NullableResult<U, E> {
         match self {
-            NullableResult::Ok(item) => NullableResult::Ok(f(item)),
-            NullableResult::Err(err) => NullableResult::Err(err),
-            NullableResult::None => NullableResult::None,
+            Ok(item) => Ok(f(item)),
+            Err(err) => Err(err),
+            None => None,
         }
     }
 
@@ -177,18 +234,70 @@ impl<T, E> NullableResult<T, E> {
     #[inline]
     pub fn map_err<U, F: FnOnce(E) -> U>(self, f: F) -> NullableResult<T, U> {
         match self {
-            NullableResult::Ok(item) => NullableResult::Ok(item),
-            NullableResult::Err(err) => NullableResult::Err(f(err)),
-            NullableResult::None => NullableResult::None,
+            Ok(item) => NullableResult::Ok(item),
+            Err(err) => NullableResult::Err(f(err)),
+            None => NullableResult::None,
         }
     }
 
     #[inline]
     pub fn result_optional_err(self) -> Result<T, Option<E>> {
         match self {
-            NullableResult::Ok(item) => Ok(item),
-            NullableResult::Err(err) => Err(Some(err)),
-            NullableResult::None => Err(None),
+            Ok(item) => Result::Ok(item),
+            Err(err) => Result::Err(Some(err)),
+            None => Result::Err(Option::None),
+        }
+    }
+
+    #[inline]
+    pub fn as_ref(&self) -> NullableResult<&T, &E> {
+        use NullableResult::*;
+        match self {
+            Ok(item) => Ok(item),
+            Err(err) => Err(err),
+            None => None,
+        }
+    }
+
+    #[inline]
+    pub fn as_mut(&mut self) -> NullableResult<&mut T, &mut E> {
+        use NullableResult::*;
+        match self {
+            Ok(item) => Ok(item),
+            Err(err) => Err(err),
+            None => None,
+        }
+    }
+
+    #[inline]
+    pub fn and<U>(self, res: NullableResult<U, E>) -> NullableResult<U, E> {
+        match self {
+            Ok(_) => res,
+            Err(err) => Err(err),
+            None => None,
+        }
+    }
+
+    #[inline]
+    pub fn and_then<U, F>(self, op: F) -> NullableResult<U, E>
+    where
+        F: FnOnce(T) -> NullableResult<U, E>,
+    {
+        match self {
+            Ok(item) => op(item),
+            Err(err) => Err(err),
+            None => None,
+        }
+    }
+}
+
+impl<T, E> NullableResult<NullableResult<T, E>, E> {
+    #[inline]
+    pub fn flatten(self) -> NullableResult<T, E> {
+        match self {
+            Ok(Ok(item)) => Ok(item),
+            Ok(Err(err)) | Err(err) => Err(err),
+            Ok(None) | None => None,
         }
     }
 }
@@ -197,9 +306,9 @@ impl<T, E> From<Result<Option<T>, E>> for NullableResult<T, E> {
     #[inline]
     fn from(res: Result<Option<T>, E>) -> Self {
         match res {
-            Ok(Some(item)) => NullableResult::Ok(item),
-            Ok(None) => NullableResult::None,
-            Err(err) => NullableResult::Err(err),
+            Result::Ok(Option::Some(item)) => Ok(item),
+            Result::Ok(Option::None) => None,
+            Result::Err(err) => Err(err),
         }
     }
 }
@@ -208,9 +317,9 @@ impl<T, E> From<NullableResult<T, E>> for Result<Option<T>, E> {
     #[inline]
     fn from(nr: NullableResult<T, E>) -> Self {
         match nr {
-            NullableResult::Ok(item) => Ok(Some(item)),
-            NullableResult::Err(err) => Err(err),
-            NullableResult::None => Ok(None),
+            Ok(item) => Result::Ok(Some(item)),
+            Err(err) => Result::Err(err),
+            None => Result::Ok(Option::None),
         }
     }
 }
@@ -219,8 +328,8 @@ impl<T, E> From<Result<T, E>> for NullableResult<T, E> {
     #[inline]
     fn from(res: Result<T, E>) -> Self {
         match res {
-            Ok(item) => NullableResult::Ok(item),
-            Err(err) => NullableResult::Err(err),
+            Result::Ok(item) => Ok(item),
+            Result::Err(err) => Err(err),
         }
     }
 }
@@ -229,9 +338,9 @@ impl<T, E> From<Option<Result<T, E>>> for NullableResult<T, E> {
     #[inline]
     fn from(opt: Option<Result<T, E>>) -> Self {
         match opt {
-            None => NullableResult::None,
-            Some(Ok(item)) => NullableResult::Ok(item),
-            Some(Err(err)) => NullableResult::Err(err),
+            Option::None => None,
+            Some(Result::Ok(item)) => Ok(item),
+            Some(Result::Err(err)) => Err(err),
         }
     }
 }
@@ -240,9 +349,9 @@ impl<T, E> From<NullableResult<T, E>> for Option<Result<T, E>> {
     #[inline]
     fn from(nr: NullableResult<T, E>) -> Self {
         match nr {
-            NullableResult::Ok(item) => Some(Ok(item)),
-            NullableResult::Err(err) => Some(Err(err)),
-            NullableResult::None => None,
+            Ok(item) => Some(Result::Ok(item)),
+            Err(err) => Some(Result::Err(err)),
+            None => Option::None,
         }
     }
 }
@@ -251,8 +360,8 @@ impl<T, E> From<Option<T>> for NullableResult<T, E> {
     #[inline]
     fn from(opt: Option<T>) -> Self {
         match opt {
-            Some(item) => NullableResult::Ok(item),
-            None => NullableResult::None,
+            Some(item) => Ok(item),
+            Option::None => None,
         }
     }
 }
@@ -260,9 +369,9 @@ impl<T, E> From<Option<T>> for NullableResult<T, E> {
 impl<T, E> From<Result<T, Option<E>>> for NullableResult<T, E> {
     fn from(res: Result<T, Option<E>>) -> Self {
         match res {
-            Ok(item) => NullableResult::Ok(item),
-            Err(Some(err)) => NullableResult::Err(err),
-            Err(None) => NullableResult::None,
+            Result::Ok(item) => Ok(item),
+            Result::Err(Some(err)) => Err(err),
+            Result::Err(Option::None) => None,
         }
     }
 }
@@ -318,7 +427,6 @@ impl<T, E, I: Iterator<Item = T>> GeneralIterExt<T, E> for I {
     where
         P: FnMut(&T) -> Result<bool, E>,
     {
-        use NullableResult::*;
         for item in self {
             return match pred(&item) {
                 Result::Err(err) => Err(err),
@@ -334,7 +442,6 @@ impl<T, E, I: Iterator<Item = T>> GeneralIterExt<T, E> for I {
     where
         F: FnMut(T) -> NullableResult<U, E>,
     {
-        use NullableResult::*;
         for item in self {
             return match f(item) {
                 Ok(item) => Ok(item),
@@ -360,7 +467,6 @@ where
     where
         P: FnMut(&T) -> Result<bool, E>,
     {
-        use NullableResult::*;
         self.try_find_map(|item| {
             let item = extract!(item);
             match pred(&item) {
@@ -419,13 +525,11 @@ where
 
     fn next(&mut self) -> Option<Self::Item> {
         match self.inner.next() {
-            None => None,
-            Some(NullableResult::None) => Some(NullableResult::None),
-            Some(NullableResult::Err(err)) => Some(NullableResult::Err(err)),
-            Some(NullableResult::Ok(item)) if (self.pred)(&item) => {
-                Some(NullableResult::Ok(item))
-            }
-            Some(NullableResult::Ok(_)) => self.next(),
+            Option::None => Option::None,
+            Some(None) => Some(None),
+            Some(Err(err)) => Some(Err(err)),
+            Some(Ok(item)) if (self.pred)(&item) => Some(Ok(item)),
+            Some(Ok(_)) => self.next(),
         }
     }
 }
@@ -456,10 +560,10 @@ where
     #[inline]
     fn next(&mut self) -> Option<Self::Item> {
         match self.inner.next() {
-            None => None,
-            Some(NullableResult::None) => Some(NullableResult::None),
-            Some(NullableResult::Err(err)) => Some(NullableResult::Err(err)),
-            Some(NullableResult::Ok(item)) => (self.f)(item),
+            Option::None => Option::None,
+            Some(None) => Some(None),
+            Some(Err(err)) => Some(Err(err)),
+            Some(Ok(item)) => (self.f)(item),
         }
     }
 }
